/**
 * Class used for driving animations.
 */
class Driver {
    constructor(timeline) {
        this.timeline = timeline

        // Driver controls
        this.slider = document.getElementById('slider')
        this.animationTimeLeft = document.getElementById('left')
        this.animationTimeRight = document.getElementById('right')
        this.playPauseButton = document.getElementById('playPause')
        this.loopButton = document.getElementById('loop')
        this.interval = setInterval(() => { this.updateSliderIfAnimating() }, 100/3)
        this.createControlFunctions()
    }

    get timeline() {
        return this._timeline
    }

    set timeline(timeline) {
        if (this._timeline != null) {
            this.pause()
        }
        this._timeline = timeline

        this.timingAnimation = document.getElementById('timer').animate({}, timeline.duration)
        this.timingAnimation.currentTime = 0
        this.timingAnimation.pause()

        this.animations = timeline.createAllAnimations()
        this.shouldPlay = true
        this.pause()
    }

    get duration() {
        return this.timeline.duration
    }

    get currentTime() {
        return this.timingAnimation.currentTime
    }

    set currentTime(time) {
        for (const animation of this.animations) {
            animation.currentTime = time
        }
        for (const shape of this.timeline.allShapes) {
            shape.setCurrentTime(time / 1000)
        }
        this.timingAnimation.currentTime = time
    }

    updateSliderIfAnimating() {
        if (this.timingAnimation.playState != "paused") {
            this.sliderValue = this.currentTime / this.timeline.duration
            this.setAnimationTimeLabels(this.currentTime)
        }
    }   

    play() {
        this.interval = setInterval(() => { this.updateSliderIfAnimating() }, 100/3)
        this.timingAnimation.play()
        for (const animation of this.animations) {
            animation.play()
        }
        for (const shape of this.timeline.allShapes) {
            var t = shape.getCurrentTime() % this.timeline.duration
            shape.setCurrentTime(t)
            shape.unpauseAnimations()
        }
    }

    isPlaying() {
        if (this.timingAnimation == null) {
            return false
        }
        return (this.timingAnimation.playState == "running")
    }

    pause() {
        clearInterval(this.interval)
        this.timingAnimation.pause()
        for (const animation of this.animations) {
            animation.pause()
        }
        for (const shape of this.timeline.allShapes) {
            shape.pauseAnimations()
        }
    }

    stop() {
        this.playPauseButton.checked = false
        this.shouldPlay = false
        this.pause()

        let d = this.duration
        this.setAnimationTimeLabels(d)
        this.sliderValue = d / this.timeline.duration - 0.001
    }

    togglePlayback() {
        if (this.playPauseButton.checked) {
            this.shouldPlay = true
            if (this.timingAnimation.playState == "finished" || 
                this.currentTime == this.timeline.duration) {
                this.currentTime = 0
            }
            this.play()
        } else {
            this.shouldPlay = false
            this.pause()
        }
    }

    //------------------
    // interface updates
    //------------------

    createControlFunctions() {
        this.playPauseButton.onchange = () => {
            this.togglePlayback()
        }
    
        this.slider.onchange = () => {
            if (this.shouldPlay) {
                this.play()
            }
        }
    
        this.timingAnimation.onfinish = () => {
            if (this.loopButton.checked) {
                this.currentTime = 0
            } else {
                this.stop()
            }
        }
    
        this.slider.oninput = () => {
            this.pause()
    
            var newTime = this.slider.value * this.duration
            this.setAnimationTimeLabels(newTime)
    
            var newTime = Math.min(newTime, this.duration - 1)
            this.currentTime = newTime
        }

        this.slider.onchange = () => {
            if (this.shouldPlay == true) {
                this.play()
            }
        }
    }

    setAnimationTimeLabels(value) {
        this.animationTimeLeft.innerHTML = Driver.convertTimeToString(value)
        this.animationTimeRight.innerHTML = Driver.convertTimeToString(this.duration - value)
    }

    get sliderValue() {
        return this.slider.value
    }

    set sliderValue(value) {
        this.slider.value = value
    }


    //---------------
    // helper methods
    //---------------

    static convertTimeToString(milliseconds) {
        var date = new Date(null)
        date.setMilliseconds(milliseconds)
        return date.toISOString().substr(14, 8);
    }
}
